unit rloop_resonnance;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls,
  Math;

type

  { TWdResonnance }

  TWdResonnance = class(TForm)
    CbxInd: TComboBox;
    CbxCap: TComboBox;
    CbxFre: TComboBox;
    EdInd: TEdit;
    EdCap: TEdit;
    EdFre: TEdit;
    Gr01: TGroupBox;
    PaFr: TPanel;
    PaIn: TPanel;
    PaCa: TPanel;
    PaR1: TPanel;
    RadFr: TRadioButton;
    RadIn: TRadioButton;
    RadCa: TRadioButton;
    procedure CbxCapChange(Sender: TObject);
    procedure CbxFreChange(Sender: TObject);
    procedure CbxIndChange(Sender: TObject);
    procedure EdCapChange(Sender: TObject);
    procedure EdFreChange(Sender: TObject);
    procedure EdIndChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RadCaChange(Sender: TObject);
    procedure RadFrChange(Sender: TObject);
    procedure RadInChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  WdResonnance: TWdResonnance;

implementation

Uses rloop_main;

{$R *.lfm}

{ TWdResonnance }

const
  ColMauve_=$00E4D6E4;
  ColVert_=$00D3F3CF;
  ColBleu_=$00E7D7C9;

var
  Inductance:real;
  Capacitance:real;
  Frequence:real;

function ItemToInteger(Select: byte): integer;
var
  Idx: integer;
begin
  Idx := -1;
  with WdResonnance do
  begin
    case Select of
      //Inductance
      1: Idx := CbxInd.ItemIndex;
      //Capacitance
      2: Idx := CbxCap.ItemIndex;
      //Fréquence
      3: Idx := CbxFre.ItemIndex;
    end;
  end;
  Result := Idx;
end;

function Unites(Select: integer; v: real): real;
var
  k: real;
  Idx: integer;
begin
  //Idx=ItemIndex
  Idx := ItemToInteger(Select);
  with WdResonnance do
  begin
    case Select of
      //Inductance
      1: case Idx of
        0: k := 1E9;  //GH
        1: k := 1E6;  //MH
        2: k := 1E3;  //kH
        3: k := 1;    //Farad
        4: k := 1E-3; //mH
        5: k := 1E-6; //µH
        6: k := 1E-9; //nH
        7: k := 1E-12;//pH
        8: k := 1E-15;//fH
      end;
      //Capacitance
      2: case Idx of
        0: k := 1E9;  //GF
        1: k := 1E6;  //MF
        2: k := 1E3;  //kF
        3: k := 1;    //Farad
        4: k := 1E-3; //mF
        5: k := 1E-6; //µF
        6: k := 1E-9; //nF
        7: k := 1E-12;//pF
        8: k := 1E-15;//fF
      end;
      //Fréquence
      3: case Idx of
        0: k := 1E9;  //Ghz
        1: k := 1E6;  //Mhz
        2: k := 1E3;  //Khz
        3: k := 1;    //Hz
        4: k := 1E-3; //mHz
        5: k := 1E-6; //µHz
        6: k := 1E-9; //nHz
        7: k := 1E-12;//pHz
        8: k := 1E-15;//fHz
      end;
    end;
  end;
  Result := v * k;
end;

function Unit_Auto(Select: integer; v: real): real;
var
  r, k: real;
begin
  r := v;
  k := 1;
  if v = 0 then
  begin
    r := 0;
  end
  else
  begin
    with WdResonnance do
    begin
      if v >= 1 then
      begin
        if v >= 1E9 then
        begin
          k := 1E-9;
          case Select of
            1: CbxInd.ItemIndex:=0;
            2: CbxCap.ItemIndex:=0;
            3: CbxFre.ItemIndex:=0;
          end;
        end
        else
        begin
          if v >= 1E6 then
          begin
            k := 1E-6;
            case Select of
              1: CbxInd.ItemIndex:=1;
              2: CbxCap.ItemIndex:=1;
              3: CbxFre.ItemIndex:=1;
            end;
          end
          else
          begin
            if v >= 1E3 then
            begin
              k := 1E-3;
              case Select of
                1: CbxInd.ItemIndex:=2;
                2: CbxCap.ItemIndex:=2;
                3: CbxFre.ItemIndex:=2;
              end;
            end
            else
            begin
              k := 1;
              case Select of
                1: CbxInd.ItemIndex:=3;
                2: Cbxcap.ItemIndex:=3;
                3: CbxFre.ItemIndex:=3;
              end;
            end;
          end;

        end;
      end
      else
      begin
        if v >= 1E-3 then
        begin
          k := 1E3;
          case Select of
            1: CbxInd.ItemIndex:=4;
            2: CbxCap.ItemIndex:=4;
            3: CbxFre.ItemIndex:=4;
          end;
        end
        else
        begin
          if v >= 1E-6 then
          begin
            k := 1E6;
            case Select of
              1: CbxInd.ItemIndex:=5;
              2: CbxCap.ItemIndex:=5;
              3: CbxFre.ItemIndex:=5;
            end;
          end
          else
          begin
            if v >= 1E-9 then
            begin
              k := 1E9;
              case Select of
                1: CbxInd.ItemIndex:=6;
                2: CbxCap.ItemIndex:=6;
                3: Cbxfre.ItemIndex:=6;
              end;
            end
            else
            begin
              if v >= 1E-12 then
              begin
                k := 1E12;
                case Select of
                  1: CbxInd.ItemIndex:=7;
                  2: CbxCap.ItemIndex:=7;
                  3: CbxFre.ItemIndex:=7;
                end;
              end
              else
              begin
                if v >= 1E-15 then
                begin
                  k := 1E15;
                  case Select of
                    1: CbxInd.ItemIndex:=8;
                    2: CbxCap.ItemIndex:=8;
                    3: CbxFre.ItemIndex:=8;
                  end;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  end;
  Result := r * k;
end;

procedure Saisie_vers_Panel(Select: integer);
var
  s: string;
begin
  with WdResonnance do
  begin
    case Select of
      //Inductance
      1:
      begin
        s := EdInd.Text;
        if WdMain.If_Chiffre(s) then
        begin
          Inductance := WdMain.StrToReal(s);
          Inductance := Unites(Select, Inductance);
        end;
      end;
      //Capacitance
      2:
      begin
        s := EdCap.Text;
        if WdMain.If_Chiffre(s) then
        begin
          Capacitance := WdMain.StrToReal(s);
          Capacitance := Unites(Select, Capacitance);
        end;
      end;
      //Fréquence
      3:
      begin
        s := EdFre.Text;
        if WdMain.If_Chiffre(s) then
        begin
          Frequence := WdMain.StrToReal(s);
          Frequence := Unites(Select, Frequence);
        end;
      end;
    end;
    Application.ProcessMessages;
  end;
end;

procedure Calcul_ICF;
var
  Af,v:real;
begin
  // 50 µH et  100 pF =   2,2448 MHz
  // 50 µH et  20 pF  =   5 MHz
  // 10 nH et 100 pF  = 159 MHz
  //520 nH et 256 pF  = 13,8 MHz
  With WdResonnance do begin
    //F = 1 / ( 2 * Pi * Sqrt(L * C))
    //F² * 4 * Pi² * L * C = 1
    //L = 1 / F² * 4 * Pi² * C
    if RadFr.Checked then begin
      if (Inductance>0) and (Capacitance>0) then begin
        Frequence:=1/(2*PI*Sqrt(Inductance*Capacitance));
      end
      else begin
        Frequence:=0;
      end;
      Af:=Unit_Auto(3,Frequence);
      EdFre.Text := WdMain.RealToStr(Af, 3);
    end else begin
      if RadIn.Checked then begin
        if (Frequence>0) and (Capacitance>0) then begin
          //L = 1 / F² * 4 * Pi² * C
          v := Power(Frequence,2) * 4 * Power(PI,2);
          Inductance := 1 /  (v * Capacitance);
        end
        else begin
          Inductance:=0;
        end;
        Af:=Unit_Auto(1,Inductance);
        EdInd.Text := WdMain.RealToStr(Af, 3);
      end else begin
        if (Frequence>0) and (Inductance>0) then begin
          // C = 1 / F² * 4 * Pi² * L
          v := Power(Frequence,2) * 4 * Power(PI,2);
          Capacitance := 1 /  (v * Inductance);
        end
        else begin
          Capacitance:=0;
        end;
        Af:=Unit_Auto(2,Capacitance);
        EdCap.Text := WdMain.RealToStr(Af, 3);
      end;
    end;
  end;
end;

procedure TWdResonnance.EdIndChange(Sender: TObject);
begin
  EdInd.Text := WdMain.Saisie_Chiffres(EdInd.Text);
  EdInd.SelStart := Length(EdInd.Text);
  Saisie_vers_Panel(1);
  Calcul_ICF;
end;

procedure TWdResonnance.EdCapChange(Sender: TObject);
begin
  Edcap.Text := WdMain.Saisie_Chiffres(EdCap.Text);
  EdCap.SelStart := Length(EdCap.Text);
  Saisie_vers_Panel(2);
  Calcul_ICF;
end;

procedure TWdResonnance.CbxIndChange(Sender: TObject);
begin
  Saisie_vers_Panel(1);
  Calcul_ICF;
end;

procedure TWdResonnance.CbxCapChange(Sender: TObject);
begin
  Saisie_vers_Panel(2);
  Calcul_ICF;
end;

procedure TWdResonnance.CbxFreChange(Sender: TObject);
begin
  Saisie_vers_Panel(3);
  Calcul_ICF;
end;

procedure TWdResonnance.EdFreChange(Sender: TObject);
begin
  EdFre.Text := WdMain.Saisie_Chiffres(EdFre.Text);
  EdFre.SelStart := Length(EdFre.Text);
  Saisie_vers_Panel(3);
  Calcul_ICF;
end;

procedure TWdResonnance.FormShow(Sender: TObject);
begin
  Inductance:=0;
  Capacitance:=0;
  Frequence:=0;
  ActiveControl:=EdInd;
end;

procedure TWdResonnance.RadCaChange(Sender: TObject);
begin
  With EdFre do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  With EdInd do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  With EdCap do begin
    Color:=ColVert_;
    ReadOnly:=true;
    Cursor:=crDefault;
  end;
  ActiveControl:=EdInd;
end;

procedure TWdResonnance.RadFrChange(Sender: TObject);
begin
  With EdFre do begin
    Color:=ColMauve_;
    ReadOnly:=true;
    Cursor:=crDefault;
  end;
  With EdInd do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  With EdCap do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  ActiveControl:=EdInd;
end;

procedure TWdResonnance.RadInChange(Sender: TObject);
begin
  With EdFre do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  With EdInd do begin
    Color:=ColBleu_;
    ReadOnly:=true;
    Cursor:=crDefault;
  end;
  With EdCap do begin
    Color:=ClWhite;
    ReadOnly:=false;
    Cursor:=crHandPoint;
  end;
  ActiveControl:=EdCap;
end;

end.

